package ctbrec.ui.settings;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import ctbrec.Config;
import ctbrec.recorder.postprocessing.PostProcessor;
import ctbrec.recorder.postprocessing.Remuxer;
import ctbrec.recorder.postprocessing.Renamer;
import ctbrec.ui.controls.Dialogs;
import ctbrec.ui.settings.api.Preferences;
import javafx.collections.ObservableList;
import javafx.scene.Scene;
import javafx.scene.layout.Region;

public class PostProcessingDialogFactory {

    static Map<Class<?>, Class<?>> ppToDialogMap = new HashMap<>();
    static {
        ppToDialogMap.put(Remuxer.class, RemuxerPaneFactory.class);
        ppToDialogMap.put(Renamer.class, RenamerPaneFactory.class);
    }

    private PostProcessingDialogFactory() {
    }

    public static void openNewDialog(PostProcessor pp, Config config, Scene scene, ObservableList<PostProcessor> stepList) {
        openDialog(pp, config, scene, stepList, true);
    }

    public static void openEditDialog(PostProcessor pp, Config config, Scene scene, ObservableList<PostProcessor> stepList) {
        openDialog(pp, config, scene, stepList, false);
    }

    private static void openDialog(PostProcessor pp, Config config, Scene scene, ObservableList<PostProcessor> stepList, boolean newEntry) {
        boolean ok;
        try {
            Preferences preferences = createPreferences(pp);
            Region view = preferences.getView(false);
            view.setMinWidth(600);
            ok = Dialogs.showCustomInput(scene, "Configure " + pp.getName(), view);
            if (ok) {
                preferences.save();
                if (newEntry) {
                    stepList.add(pp);
                }
            }
        } catch (NoSuchMethodException | SecurityException | IllegalAccessException | IllegalArgumentException | InvocationTargetException
                | InstantiationException | IOException e) {
            Dialogs.showError("New post-processing step", "Couldn't create dialog for " + pp.getName(), e);
        }
    }

    private static Preferences createPreferences(PostProcessor pp) throws InstantiationException, IllegalAccessException, IllegalArgumentException,
    InvocationTargetException, NoSuchMethodException, SecurityException {
        Class<?> paneFactoryClass = ppToDialogMap.get(pp.getClass());
        AbstractPostProcessingPaneFactory factory = (AbstractPostProcessingPaneFactory) paneFactoryClass.getDeclaredConstructor().newInstance();
        return factory.createPostProcessorPane(pp);
    }
}
